@@ -22,6 +22,7 @@ this module can be modified.
 import contextlib
 import datetime
 import inspect
+import os
 import threading
 import warnings
 from enum import Enum, IntEnum, unique
@@ -609,13 +610,21 @@ Set this to None to disable this behaviour entirely.
 )
 
 
+def is_in_ci() -> bool:
+    # GitHub Actions, Travis CI and AppVeyor have "CI"
+    # Azure Pipelines has "TF_BUILD"
+    return "CI" in os.environ or "TF_BUILD" in os.environ
+
+
 settings._define_setting(
     "print_blob",
-    default=False,
+    default=is_in_ci(),
+    show_default=False,
     options=(True, False),
     description="""
 If set to ``True``, Hypothesis will print code for failing examples that can be used with
 :func:`@reproduce_failure <hypothesis.reproduce_failure>` to reproduce the failing example.
+The default is ``True`` if the ``CI`` or ``TF_BUILD`` env vars are set, ``False`` otherwise.
 """,
 )
 
