@@ -18,7 +18,6 @@ import importlib
 import inspect
 import platform
 import sys
-import time
 import typing
 
 PYPY = platform.python_implementation() == "PyPy"
@@ -50,10 +49,6 @@ def int_to_byte(i):
     return bytes([i])
 
 
-def benchmark_time():
-    return time.monotonic()
-
-
 def a_good_encoding():
     return "utf-8"
 
@@ -97,7 +92,7 @@ def is_typed_named_tuple(cls):
     return (
         issubclass(cls, tuple)
         and hasattr(cls, "_fields")
-        and hasattr(cls, "_field_types")
+        and (hasattr(cls, "_field_types") or hasattr(cls, "__annotations__"))
     )
 
 
@@ -113,26 +108,57 @@ if sys.version_info[:2] < (3, 6):
             thing = thing.__init__  # type: ignore
         try:
             spec = inspect.getfullargspec(thing)
-            return {
+            hints = {
                 k: v
                 for k, v in spec.annotations.items()
                 if k in (spec.args + spec.kwonlyargs) and isinstance(v, type)
             }
         except TypeError:
-            return {}
+            hints = {}
+        try:
+            hints["return"] = typing.get_type_hints(thing)["return"]
+        except Exception:
+            pass
+        return hints
 
 
 else:
 
     def get_type_hints(thing):
-        if inspect.isclass(thing) and not hasattr(thing, "__signature__"):
-            if is_typed_named_tuple(thing):
-                # Special handling for typing.NamedTuple
-                return thing._field_types  # type: ignore
-            thing = thing.__init__  # type: ignore
+        """Like the typing version, but tries harder and never errors.
+
+        Tries harder: if the thing to inspect is a class but typing.get_type_hints
+        raises an error or returns no hints, then this function will try calling it
+        on the __init__ method.  This second step often helps with user-defined
+        classes on older versions of Python.
+
+        Never errors: instead of raising TypeError for uninspectable objects, or
+        NameError for unresolvable forward references, just return an empty dict.
+        """
         try:
-            return typing.get_type_hints(thing)
-        except TypeError:
+            hints = {}
+            if inspect.isclass(thing) and hasattr(thing, "__signature__"):
+                # It is possible for the signature and annotations attributes to
+                # differ on an object due to renamed arguments.
+                # To prevent missing arguments we use the signature to provide any type
+                # hints it has and then override any common names with the more
+                # comprehensive type information from get_type_hints
+                # See https://github.com/HypothesisWorks/hypothesis/pull/2580
+                # for more details.
+                spec = inspect.getfullargspec(thing)
+                hints = {
+                    k: v
+                    for k, v in spec.annotations.items()
+                    if k in (spec.args + spec.kwonlyargs) and isinstance(v, type)
+                }
+            hints.update(typing.get_type_hints(thing))
+        except (AttributeError, TypeError, NameError):
+            hints = {}
+        if hints or not inspect.isclass(thing):
+            return hints
+        try:
+            return typing.get_type_hints(thing.__init__)
+        except (TypeError, NameError, AttributeError):
             return {}
 
 
@@ -211,21 +237,12 @@ def ceil(x):
     return y
 
 
-try:
-    from django.test import TransactionTestCase
-
-    def bad_django_TestCase(runner):
-        if runner is None:
-            return False
-        if not isinstance(runner, TransactionTestCase):
-            return False
-
-        from hypothesis.extra.django._impl import HypothesisTestCase
-
-        return not isinstance(runner, HypothesisTestCase)
-
-
-except Exception:
-    # Can't use ImportError, because of e.g. Django config errors
-    def bad_django_TestCase(runner):
+def bad_django_TestCase(runner):
+    if runner is None or "django.test" not in sys.modules:
         return False
+    if not isinstance(runner, sys.modules["django.test"].TransactionTestCase):
+        return False
+
+    from hypothesis.extra.django._impl import HypothesisTestCase
+
+    return not isinstance(runner, HypothesisTestCase)
