@@ -26,6 +26,7 @@ import os
 import sys
 import typing
 import uuid
+from pathlib import PurePath
 from types import FunctionType
 
 from hypothesis import strategies as st
@@ -131,14 +132,41 @@ def is_generic_type(type_):
     )
 
 
+def _try_import_forward_ref(thing, bound):  # pragma: no cover
+    """
+    Tries to import a real bound type from ``TypeVar`` bound to a ``ForwardRef``.
+
+    This function is very "magical" to say the least, please don't use it.
+    This function fully covered, but is excluded from coverage
+    because we can only cover each path in a separate python version.
+    """
+    try:
+        return typing._eval_type(bound, vars(sys.modules[thing.__module__]), None)
+    except (KeyError, AttributeError, NameError):
+        if (
+            isinstance(thing, typing.TypeVar)
+            and getattr(thing, "__module__", None) == "typing"
+        ):
+            raise ResolutionFailed(
+                "Looks like you are using python3.6 or any of the previous versions "
+                "together with a TypeVar bound to a ForwardRef. "
+                "It is not supported. Upgrading to python3.7+ will solve this problem."
+            ) from None
+        # We fallback to `ForwardRef` instance, you can register it as a type as well:
+        # >>> from typing import ForwardRef
+        # >>> from hypothesis import strategies as st
+        # >>> st.register_type_strategy(ForwardRef('YourType'), your_strategy)
+        return bound
+
+
 def from_typing_type(thing):
     # We start with special-case support for Union and Tuple - the latter
     # isn't actually a generic type. Then we handle Literal since it doesn't
-    # support `isinstance`. Support for Callable may be added to this section
-    # later.
+    # support `isinstance`.
+    #
     # We then explicitly error on non-Generic types, which don't carry enough
     # information to sensibly resolve to strategies at runtime.
-    # Finally, we run a variation of the subclass lookup in st.from_type
+    # Finally, we run a variation of the subclass lookup in `st.from_type`
     # among generic types in the lookup.
     #
     # Under 3.6 Union is handled directly in st.from_type, as the argument is
@@ -177,25 +205,6 @@ def from_typing_type(thing):
             else:
                 literals.append(arg)
         return st.sampled_from(literals)
-    if isinstance(thing, typing.TypeVar):
-        if getattr(thing, "__bound__", None) is not None:
-            strat = unwrap_strategies(st.from_type(thing.__bound__))
-            if not isinstance(strat, OneOfStrategy):
-                return strat
-            # The bound was a union, or we resolved it as a union of subtypes,
-            # so we need to unpack the strategy to ensure consistency across uses.
-            # This incantation runs a sampled_from over the strategies inferred for
-            # each part of the union, wraps that in shared so that we only generate
-            # from one type per testcase, and flatmaps that back to instances.
-            return st.shared(
-                st.sampled_from(strat.original_strategies), key="typevar=%r" % (thing,)
-            ).flatmap(lambda s: s)
-        if getattr(thing, "__constraints__", None):
-            return st.shared(
-                st.sampled_from(thing.__constraints__), key="typevar=%r" % (thing,)
-            ).flatmap(st.from_type)
-        # Constraints may be None or () on various Python versions.
-        return st.text()  # An arbitrary type for the typevar
     # Now, confirm that we're dealing with a generic type as we expected
     if not isinstance(thing, typing_root_type):  # pragma: no cover
         raise ResolutionFailed("Cannot resolve %s to a strategy" % (thing,))
@@ -353,6 +362,8 @@ _global_type_lookup[type] = st.sampled_from(
     [type(None)] + sorted(_global_type_lookup, key=str)
 )
 
+if sys.version_info[:2] >= (3, 6):  # pragma: no branch
+    _global_type_lookup[os.PathLike] = st.builds(PurePath, st.text())
 if sys.version_info[:2] >= (3, 9):  # pragma: no cover
     # subclass of MutableMapping, and in Python 3.9 we resolve to a union
     # which includes this... but we don't actually ever want to build one.
@@ -571,3 +582,34 @@ def resolve_Callable(thing):
         like=(lambda: None) if len(thing.__args__) == 1 else (lambda *a, **k: None),
         returns=st.from_type(thing.__args__[-1]),
     )
+
+
+@register(typing.TypeVar)
+def resolve_TypeVar(thing):
+    type_var_key = "typevar=%r" % (thing,)
+
+    if getattr(thing, "__bound__", None) is not None:
+        bound = thing.__bound__
+        if isinstance(bound, ForwardRef):
+            bound = _try_import_forward_ref(thing, bound)
+        strat = unwrap_strategies(st.from_type(bound))
+        if not isinstance(strat, OneOfStrategy):
+            return strat
+        # The bound was a union, or we resolved it as a union of subtypes,
+        # so we need to unpack the strategy to ensure consistency across uses.
+        # This incantation runs a sampled_from over the strategies inferred for
+        # each part of the union, wraps that in shared so that we only generate
+        # from one type per testcase, and flatmaps that back to instances.
+        return st.shared(
+            st.sampled_from(strat.original_strategies), key=type_var_key,
+        ).flatmap(lambda s: s)
+
+    builtin_scalar_types = [type(None), bool, int, float, str, bytes]
+    return st.shared(
+        st.sampled_from(
+            # Constraints may be None or () on various Python versions.
+            getattr(thing, "__constraints__", None)
+            or builtin_scalar_types,
+        ),
+        key=type_var_key,
+    ).flatmap(st.from_type)
